/**
 *    Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.squeakysand.devtools.checkstyle;

import com.puppycrawl.tools.checkstyle.api.Check;
import com.puppycrawl.tools.checkstyle.api.DetailAST;
import com.puppycrawl.tools.checkstyle.api.TokenTypes;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This class check the JUnitTestCase package name according to the test package standard.
 */
public class JUnitTestPackageName extends Check {

    private static final Log LOG = LogFactory.getLog(JUnitTestPackageName.class);
    private StringBuffer sb = new StringBuffer();

    /**
     * Selects only the Package, import and extends clauses for validating.
     *
     * @return  tokens int array which contains the index of the TokensType
     *
     * @see     com.puppycrawl.tools.checkstyle.api.Check#getDefaultTokens()
     */
    public int[] getDefaultTokens() {

        return new int[] {TokenTypes.PACKAGE_DEF, TokenTypes.IMPORT, TokenTypes.EXTENDS_CLAUSE};
    }

    /**
     * This method will be called whenever checkstyle framework parses the java sorce files containg
     * above selected clauses.
     *
     * @param  ast  Object which contains the TokenTypes
     */

    public void visitToken(DetailAST ast) {

        if (ast.getType() == TokenTypes.PACKAGE_DEF) {
            logMsg("PACKAGE_DEF type is identified");
            sb.append(ast.toStringTree());
        } else if (ast.getType() == TokenTypes.IMPORT) {
            logMsg("IMPORT type is identified");
            sb.append(ast.toStringTree());
        } else if (ast.getType() == TokenTypes.EXTENDS_CLAUSE) {
            logMsg("EXTENDS_CLAUSE type is identified");
            sb.append(ast.toStringTree());
            validate(sb, ast);
        }
    }

    /**
     * Logs the lof message only if the debug is enabled.
     *
     * @param  msg  Message to be logged
     */
    private void logMsg(String msg) {
        if (LOG.isDebugEnabled()) {
            LOG.debug(msg);
        }

    }

    /**
     * Validates the Java source files for the JUnit package structures.
     *
     * @param  sb   StringBuffer containing the source files details about package, import statement
     *              and Extends clauses
     * @param  ast  Object containing the TokenTypes
     */
    private void validate(StringBuffer sb, DetailAST ast) {
        Object[] kk = {};

        if ((sb.indexOf("TestCase") != -1) && (sb.indexOf("junit") != -1)) {
            logMsg("JUnit class identified");

            if (sb.indexOf("test") == -1) {  // checks for the package name
                log(ast.getLineNo(), "Test module package name should start with \"test\" ", kk);
            }
        } else {
            logMsg("non JUnit test class");
        }
    }

}
